@@ -52,13 +52,15 @@ import functools
 import timeit
 import sys
 import operator
+import collections
+import inspect
 
 # optunity imports
 from . import functions as fun
 from . import solvers
-from . import solver_registry
-from . import parallel as par
+from .solvers import solver_registry
 from .util import DocumentedNamedTuple as DocTup
+from .constraints import wrap_constraints
 
 
 def _manual_lines(solver_name=None):
@@ -78,6 +80,14 @@ def _manual_lines(solver_name=None):
         return solver_registry.manual(), solver_registry.solver_names()
 
 
+def available_solvers():
+    """Returns a list of all available solvers.
+
+    These can be used in :func:`optunity.make_solver`.
+    """
+    return solver_registry.solver_names()
+
+
 def manual(solver_name=None):
     """Prints the manual of requested solver.
 
@@ -88,7 +98,7 @@ def manual(solver_name=None):
     if solver_name:
         man = solver_registry.get(solver_name).desc_full
     else:
-        man = solver_registry._manual_lines()
+        man = solver_registry.manual()
     print('\n'.join(man))
 
 
@@ -141,13 +151,16 @@ def suggest_solver(num_evals=50, solver_name=None, **kwargs):
     return suggestion
 
 
-def maximize(f, num_evals=50, solver_name=None, **kwargs):
+def maximize(f, num_evals=50, solver_name=None, pmap=map, **kwargs):
     """Basic function maximization routine. Maximizes ``f`` within
     the given box constraints.
 
     :param f: the function to be maximized
     :param num_evals: number of permitted function evaluations
     :param solver_name: name of the solver to use (optional)
+    :type solver_name: string
+    :param pmap: the map function to use
+    :type pmap: callable
     :param kwargs: box constraints, a dict of the following form
         ``{'parameter_name': [lower_bound, upper_bound], ...}``
     :returns: retrieved maximum, extra information and solver info
@@ -160,21 +173,25 @@ def maximize(f, num_evals=50, solver_name=None, **kwargs):
     assert all([len(v) == 2 and v[0] < v[1]
                 for v in kwargs.values()]), 'Box constraints improperly specified: should be [lb, ub] pairs'
 
-    func = _wrap_hard_box_constraints(f, kwargs, sys.float_info.min)
+    f = _wrap_hard_box_constraints(f, kwargs, sys.float_info.min)
 
     suggestion = suggest_solver(num_evals, solver_name, **kwargs)
     solver = make_solver(**suggestion)
-    solution, details = optimize(solver, func, maximize=True, max_evals=num_evals)
+    solution, details = optimize(solver, f, maximize=True, max_evals=num_evals,
+                                 pmap=pmap)
     return solution, details, suggestion
 
 
-def minimize(f, num_evals=50, solver_name=None, **kwargs):
+def minimize(f, num_evals=50, solver_name=None, pmap=map, **kwargs):
     """Basic function minimization routine. Minimizes ``f`` within
     the given box constraints.
 
     :param f: the function to be minimized
     :param num_evals: number of permitted function evaluations
     :param solver_name: name of the solver to use (optional)
+    :type solver_name: string
+    :param pmap: the map function to use
+    :type pmap: callable
     :param kwargs: box constraints, a dict of the following form
         ``{'parameter_name': [lower_bound, upper_bound], ...}``
     :returns: retrieved minimum, extra information and solver info
@@ -191,25 +208,29 @@ def minimize(f, num_evals=50, solver_name=None, **kwargs):
 
     suggestion = suggest_solver(num_evals, solver_name, **kwargs)
     solver = make_solver(**suggestion)
-    solution, details = optimize(solver, func, maximize=False, max_evals=num_evals)
+    solution, details = optimize(solver, func, maximize=False, max_evals=num_evals,
+                                 pmap=pmap)
     return solution, details, suggestion
 
 
 def optimize(solver, func, maximize=True, max_evals=0, pmap=map):
     """Optimizes func with given solver.
 
+    :param solver: the solver to be used, for instance a result from :func:`optunity.make_solver`
+    :param func: the objective function
+    :type func: callable
+    :param maximize: maximize or minimize?
+    :type maximize: bool
+    :param max_evals: maximum number of permitted function evaluations
+    :type max_evals: int
+    :param pmap: the map() function to use, to vectorize use :func:`optunity.parallel.pmap`
+    :type pmap: function
+
     Returns the solution and a namedtuple with further details.
     Please refer to docs of optunity.maximize_results
     and optunity.maximize_stats.
 
-    Raises KeyError if
-    - ``solver_name`` is not registered
-    - ``solver_config`` is invalid to instantiate ``solver_name``
-
     """
-    ## FIXME: negating function values is fragile, since the function
-    # may already be logged by the user, in which case the log will
-    # be polluted with negated function values
 
     if max_evals > 0:
         f = fun.max_evals(max_evals)(func)
@@ -233,13 +254,13 @@ def optimize(solver, func, maximize=True, max_evals=0, pmap=map):
         solution = operator.itemgetter(index)(f.call_log.keys())._asdict()
     time = timeit.default_timer() - time
 
-    optimum = f(**solution)
+    optimum = f.call_log.get(**solution)
     num_evals += len(f.call_log)
 
     # use namedtuple to enforce uniformity in case of changes
     stats = optimize_stats(num_evals, time)
 
-    call_dict = fun.call_log2dict(f.call_log)
+    call_dict = f.call_log.to_dict()
     return solution, optimize_results(optimum, stats._asdict(),
                                       call_dict, report)
 
@@ -247,6 +268,16 @@ def optimize(solver, func, maximize=True, max_evals=0, pmap=map):
 optimize.__doc__ = '''
 Optimizes func with given solver.
 
+:param solver: the solver to be used, for instance a result from :func:`optunity.make_solver`
+:param func: the objective function
+:type func: callable
+:param maximize: maximize or minimize?
+:type maximize: bool
+:param max_evals: maximum number of permitted function evaluations
+:type max_evals: int
+:param pmap: the map() function to use, to vectorize use :func:`optunity.pmap`
+:type pmap: function
+
 Returns the solution and a ``namedtuple`` with further details.
 ''' + optimize_results.__doc__ + optimize_stats.__doc__
 
@@ -254,6 +285,14 @@ Returns the solution and a ``namedtuple`` with further details.
 def make_solver(solver_name, *args, **kwargs):
     """Creates a Solver from given parameters.
 
+    :param solver_name: the solver to instantiate
+    :type solver_name: string
+    :param args: positional arguments to solver constructor.
+    :param kwargs: keyword arguments to solver constructor.
+
+    Use :func:`optunity.manual` to get a list of registered solvers.
+    For constructor arguments per solver, please refer to :doc:`/user/solvers`.
+
     Raises ``KeyError`` if
 
     - ``solver_name`` is not registered
@@ -267,9 +306,12 @@ def make_solver(solver_name, *args, **kwargs):
 def wrap_call_log(f, call_dict):
     """Wraps an existing call log (as dictionary) around f.
 
+    This allows you to communicate known function values to solvers.
+    (currently available solvers do not use this info)
+
     """
     f = fun.logged(f)
-    call_log = fun.dict2call_log(call_dict)
+    call_log = fun.CallLog.from_dict(call_dict)
     if f.call_log:
         f.call_log.update(call_log)
     else:
@@ -277,64 +319,17 @@ def wrap_call_log(f, call_dict):
     return f
 
 
-def wrap_constraints(f, default=None, **kwargs):
-    """Decorates f with all constraints listed in kwargs.
-
-    constraint_dict may have the following keys:
-
-    - ``ub_?``: upper bound
-    - ``lb_?``: lower bound
-    - ``range_??``: range'
-
-    where '?' can be either 'o' (open) or 'c' (closed).
-    The values of constraint_dict are dicts with argname-value pairs.
-
-    """
-    if not kwargs:
-        return f
-
-    # jump table to get the right constraint function
-    jt = {'ub_o': fun.constr_ub_o,
-          'ub_c': fun.constr_ub_c,
-          'lb_o': fun.constr_lb_o,
-          'lb_c': fun.constr_lb_c,
-          'range_oo': fun.constr_range_oo,
-          'range_oc': fun.constr_range_oc,
-          'range_co': fun.constr_range_co,
-          'range_cc': fun.constr_range_cc}
-
-    # construct constraint list
-    constraints = []
-    for constr_name, pars in kwargs.items():
-        constr_fun = jt[constr_name]
-        for field, bounds in pars.items():
-            constraints.append(functools.partial(constr_fun,
-                                                 field=field,
-                                                 bounds=bounds))
-
-    # wrap function
-    if default is None:
-        @fun.constrained(constraints)
-        @functools.wraps(f)
-        def func(*args, **kwargs):
-            return f(*args, **kwargs)
-    else:
-        @fun.violations_defaulted(default)
-        @fun.constrained(constraints)
-        @functools.wraps(f)
-        def func(*args, **kwargs):
-            return f(*args, **kwargs)
-    return func
-
-
 def _wrap_hard_box_constraints(f, box, default):
     """Places hard box constraints on the domain of ``f``
     and defaults function values if constraints are violated.
 
     :param f: the function to be wrapped with constraints
-    :param box: the box, as a dict: ``{'param_name': [lb, ub], ...}
+    :type f: callable
+    :param box: the box, as a dict: ``{'param_name': [lb, ub], ...}``
+    :type box: dict
     :param default: function value to default to when constraints
         are violated
+    :type default: number
 
     """
     return wrap_constraints(f, default, range_oo=box)
