@@ -38,6 +38,9 @@ The main functions in this module are:
 * :func:`generate_folds`
 * :func:`strata_by_labels`
 * :func:`random_permutation`
+* :func:`mean`
+* :func:`identity`
+* :func:`list_mean`
 
 .. moduleauthor:: Marc Claesen
 
@@ -47,17 +50,36 @@ import math
 import random
 import itertools as it
 import functools
-import collections
 import operator as op
+import array
+import inspect
 
 
-__all__ = ['select', 'random_permutation', 'map_clusters', 'cross_validated',
-           'generate_folds', 'strata_by_labels']
+__all__ = ['select', 'random_permutation', 'cross_validated',
+           'generate_folds', 'strata_by_labels', 'mean', 'identity',
+           'list_mean']
+
+_spark_available = True
+try:
+    import pyspark
+except ImportError:
+    _spark_available = False
 
 
 def select(collection, indices):
-    """Selects the subset specified by indices from collection."""
-    return [collection[i] for i in indices]
+    """Selects the subset specified by indices from collection.
+
+    >>> select([0, 1, 2, 3, 4], [1, 3])
+    [1, 3]
+
+    """
+    try:
+        return collection[indices, ...]
+    except TypeError: # not dealing with numpy or comparable, probably a list
+        if _spark_available and isinstance(collection, pyspark.rdd.RDD):
+            indexset = set(indices)
+            return collection.zipWithIndex().filter(lambda x: x[1] in indexset).map(lambda x: x[0])
+        return [collection[i] for i in indices]
 
 
 # https://docs.python.org/2/library/itertools.html#itertools.permutations
@@ -74,20 +96,6 @@ def random_permutation(data):
     return d
 
 
-def map_clusters(clusters):
-    """Maps data instance indices to cluster indices.
-
-    :param clusters: list of lists of instance indices
-    :returns: a dictionary mapping instance indices to clusters
-    """
-    idx2cluster = collections.defaultdict(collections.deque)
-    if clusters:
-        for cluster, indices in enumerate(clusters):
-            for index in indices:
-                idx2cluster[index].append(cluster)
-    return idx2cluster
-
-
 def strata_by_labels(labels):
     """Constucts a list of strata (lists) based on unique values of ``labels``.
 
@@ -97,10 +105,23 @@ def strata_by_labels(labels):
     return [list(zip(*g)[0])
             for _, g in it.groupby(enumerate(labels), op.itemgetter(1))]
 
+def _fold_sizes(num_rows, num_folds):
+    """Generates fold sizes to partition specified number of rows into number of folds.
 
-# TODO: implement support for clusters
-def generate_folds(num_rows, num_folds=10, strata=None, clusters=None,
-                   idx2cluster=None):
+    :param num_rows: number of rows (instances)
+    :type num_rows: integer
+    :param num_folds: number of folds
+    :type num_folds: integer
+    :returns: an array.array of fold sizes (of length num_folds)
+
+    """
+    sizes = array.array('i', [0] * num_folds)
+    for i in range(num_folds):
+        sizes[i] = int(math.floor(float(num_rows - sum(sizes)) / (num_folds - i)))
+    return sizes
+
+
+def generate_folds(num_rows, num_folds=10, strata=None, clusters=None):
     """Generates folds for a given number of rows.
 
     :param num_rows: number of data instances
@@ -111,63 +132,125 @@ def generate_folds(num_rows, num_folds=10, strata=None, clusters=None,
     :param clusters: (optional) list of lists indicating clustered instances.
         Clustered instances must be placed in a single fold to avoid
         information leaks.
-    :param idx2cluster: (optional) mapping of instance indices to cluster ids.
     :returns: a list of folds, each fold is a list of instance indices
 
+    >>> folds = generate_folds(num_rows=6, num_folds=2, clusters=[[1, 2]], strata=[[3,4]])
+    >>> len(folds)
+    2
+    >>> i1 = [idx for idx, fold in enumerate(folds) if 1 in fold]
+    >>> i2 = [idx for idx, fold in enumerate(folds) if 2 in fold]
+    >>> i1 == i2
+    True
+    >>> i3 = [idx for idx, fold in enumerate(folds) if 3 in fold]
+    >>> i4 = [idx for idx, fold in enumerate(folds) if 4 in fold]
+    >>> i3 == i4
+    False
+
     .. warning::
-        Returned folds are not necessarily of the same size, but should be
-        comparable. Size differences may grow for large numbers of (small) strata.
+        Instances in strata are not necessarily spread out over all folds. Some
+        folds may already be full due to clusters. This effect should be negligible.
+
     """
 
-    # FIXME: current partitioning has some issues
-    # fold sizes not necessarily (close to) equal
-    # if many small strata exist
-    def get_folds(rows, num_folds, require=False):
-        """Partitions rows in num_folds.
+    # sizes per fold and initialization of folds
+    sizes = _fold_sizes(num_rows, num_folds)
+    folds = [[] for _ in range(num_folds)]
 
-        If require is true, len(rows) > num_folds or an exception is raised.
-        """
-        if require and num_folds > len(rows):
-            raise ValueError
+    # the folds that still need to be filled
+    fill_queue = set(range(num_folds))
 
-        folds = [None] * num_folds
-        idx = random_permutation(rows)
-        for fold in range(num_folds):
-            fold_size = int(len(idx) / (num_folds - fold))
-            folds[fold] = idx[:fold_size]
-            del idx[:fold_size]
+    # the instances that still need to be assigned
+    instances = set(range(num_rows))
 
-        # permute so the largest folds are not always at the back
-        return random_permutation(folds)
+    if not strata:
+        strata = []
 
-    if clusters and not idx2cluster:
-        idx2cluster = map_clusters(clusters)
+    if not clusters:
+        clusters = []
 
-    if strata:  # stratified cross-validation
-        ## keep lists per stratum + 1 extra for no-care instances
-        permuted_strata = map(random_permutation, strata)
-        nocares = list(set(range(num_rows)) - set(it.chain(*strata)))
+    # instances not in any stratum/cluster are treated as a final stratum
+    if strata:
+        assigned = set(it.chain(*strata))
+    else:
+        assigned = set()
+    if clusters:
+        assigned.update(it.chain(*clusters))
 
-        # we  do not require points of each stratum in each fold,
-        # since strata can be smaller than number of folds
-        folds_per_stratum = map(lambda x: get_folds(x, num_folds, False),
-                                permuted_strata)
-        # nocare points need not be in every fold
-        folds_nocare = get_folds(nocares, num_folds, False)
+    if assigned:
+        strata.append(list(filter(lambda x: x not in assigned, instances)))
+    else:
+        strata.append(list(instances))
 
-        # merge folds across strata
-        folds_per_stratum.append(folds_nocare)
-        folds = [list(it.chain(*x)) for x in zip(*folds_per_stratum)]
+    if clusters:
+        # sort clusters by size
+        sorted_cluster_indices, _ = zip(*sorted(enumerate(clusters),
+                                                key=lambda x: len(x[1]),
+                                                reverse=True))
 
-    else:  # no stratification required
-        folds = get_folds(range(num_rows), num_folds, True)
+        # assign clusters
+        for cluster_idx in sorted_cluster_indices:
+            # retrieve eligible folds: folds that will not surpass maximum
+            # when we assign given cluster to them
+            cluster = clusters[cluster_idx]
+            cluster_size = len(cluster)
+            eligible = list(filter(lambda x: len(folds[x]) + cluster_size <= sizes[x],
+                                   fill_queue))
+
+            if not eligible:
+                raise ValueError('Unable to assign all clusters to folds.')
+
+            # choose a fold at random
+            fold_idx = random.choice(eligible)
+            folds[fold_idx].extend(cluster)
+
+            # update instances to-be-assigned
+            instances.difference_update(cluster)
+
+            # remove fold from fill_queue if it is full
+            if len(folds[fold_idx]) >= sizes[fold_idx]:
+                fill_queue.remove(fold_idx)
+
+    # assign strata
+    for stratum in strata:
+        permuted_stratum = random_permutation(list(filter(lambda x: x in
+                                                          instances, stratum[:])))
+        while permuted_stratum:
+            eligible = list(filter(lambda x: len(folds[x]) < sizes[x], fill_queue))
+
+            if not eligible:
+                raise ValueError('Unable to assign all instances to folds.')
+
+            eligible = random_permutation(eligible)
+            for instance_idx, fold_idx in zip(permuted_stratum[:], eligible):
+                folds[fold_idx].append(instance_idx)
+                if len(folds[fold_idx]) >= sizes[fold_idx]:
+                    fill_queue.remove(fold_idx)
+                instances.remove(instance_idx)
+
+            permuted_stratum = permuted_stratum[len(eligible):]
 
     return folds
 
-
 def mean(x):
     return float(sum(x)) / len(x)
 
+def list_mean(list_of_measures):
+    """Computes means of consequent elements in given list.
+
+    :param list_of_measures: a list of tuples to compute means from
+    :type list_of_measures: list
+    :returns: a list containing the means
+
+    This function can be used as an aggregator in :func:`cross_validated`,
+    when multiple performance measures are being returned by the wrapped function.
+
+    >>> list_mean([(1, 4), (2, 5), (3, 6)])
+    [2.0, 5.0]
+
+    """
+    return list(map(mean, zip(*list_of_measures)))
+
+
 def identity(x):
     return x
 
@@ -184,39 +267,44 @@ class cross_validated_callable(object):
         Strata signify instances that must be spread across folds.
         Not every instance must be in a stratum.
         Specify strata as a list of lists of instance indices.
-    :param folds: (optional) prespecified cross-validation folds to be used (list of lists).
+    :param folds: (optional) prespecified cross-validation folds to be used (list of lists (iterations) of lists (folds)).
     :param num_iter: (optional) number of iterations to use (default 1)
     :param regenerate_folds: (optional) whether or not to regenerate folds on every evaluation (default false)
     :param clusters: (optional) clusters to account for when generating folds.
         Clusters signify instances that must be assigned to the same fold.
         Not every instance must be in a cluster.
         Specify clusters as a list of lists of instance indices.
-    :param reduce: function to aggregate scores of different folds (default: mean)
+    :param aggregator: function to aggregate scores of different folds (default: mean)
 
     Use :func:`cross_validated` to create instances of this class.
     """
     def __init__(self, f, x, num_folds=10, y=None, strata=None, folds=None,
                  num_iter=1, regenerate_folds=False, clusters=None,
-                 reduce=mean):
+                 aggregator=mean):
         self._x = x
         self._y = y
         self._strata = strata
         self._clusters = clusters
-        self._idx2cluster = map_clusters(clusters)
-        # TODO: sanity check between strata & clusters? define what is allowed
         self._regenerate_folds = regenerate_folds
         self._f = f
-        self._reduce = reduce
+        self._reduce = aggregator
         if folds:
             assert (len(folds) == num_iter), 'Number of fold sets does not equal num_iter.'
-            assert (len(folds[0] == num_folds)), 'Number of folds does not match num_folds.'
+            assert (len(folds[0]) == num_folds), 'Number of folds does not match num_folds.'
             self._folds = folds
         else:
-            self._folds = [generate_folds(len(x), num_folds, self.strata,
-                                          self.clusters, self.idx2cluster)
+            self._folds = [generate_folds(self.len_x, num_folds, self.strata, self.clusters)
                            for _ in range(num_iter)]
         functools.update_wrapper(self, f)
 
+    @property
+    def len_x(self):
+        """ Number of samples in x """
+        if _spark_available and isinstance(self._x, pyspark.rdd.RDD):
+            return self._x.count()
+        return len(self._x)
+
+
     @property
     def reduce(self):
         """The aggregation function."""
@@ -242,14 +330,6 @@ class cross_validated_callable(object):
         """Clusters that were used to compute folds."""
         return self._clusters
 
-    @property
-    def idx2cluster(self):
-        """A mapping of data instances to cluster ids.
-
-        cluster ids are stored in a list, since single
-        instances may occur in several clusters."""
-        return self._idx2cluster
-
     @property
     def x(self):
         """The data that is used for cross-validation."""
@@ -276,8 +356,20 @@ class cross_validated_callable(object):
         return self._regenerate_folds
 
     def __call__(self, *args, **kwargs):
+        if args:
+            # called with positionals, we must translate them to kwargs of f
+            # otherwise positionals mess up with bounded arguments
+            argspec = inspect.getargspec(self.f).args
+            argspec.remove('x_train')
+            argspec.remove('x_test')
+            if self.y is not None:
+                argspec.remove('y_train')
+                argspec.remove('y_test')
+            for argname, arg in zip(argspec, args):
+                kwargs[argname] = arg
+
         if self.regenerate_folds:
-            self._folds = [generate_folds(len(x), self.num_folds, self.strata)
+            self._folds = [generate_folds(self.len_x, self.num_folds, self.strata)
                            for _ in range(self.num_iter)]
         scores = []
         for folds in self.folds:
@@ -291,7 +383,7 @@ class cross_validated_callable(object):
                 if not self.y is None:  # dealing with a supervised algorithm
                     kwargs['y_train'] = select(self.y, rows_train)
                     kwargs['y_test'] = select(self.y, rows_test)
-                scores.append(self.f(*args, **kwargs))
+                scores.append(self.f(**kwargs))
         return self.reduce(scores)
 
 
@@ -307,7 +399,7 @@ def cross_validated(x, num_folds=10, y=None, strata=None, folds=None, num_iter=1
         Strata signify instances that must be spread across folds.
         Not every instance must be in a stratum.
         Specify strata as a list of lists of instance indices.
-    :param folds: (optional) prespecified cross-validation folds to be used (list of lists).
+    :param folds: (optional) prespecified cross-validation folds to be used (list of lists (iterations) of lists (folds)).
     :param num_iter: (optional) number of iterations to use (default 1)
     :param regenerate_folds: (optional) whether or not to regenerate folds on every evaluation (default false)
     :param clusters: (optional) clusters to account for when generating folds.
@@ -316,39 +408,41 @@ def cross_validated(x, num_folds=10, y=None, strata=None, folds=None, num_iter=1
         Specify clusters as a list of lists of instance indices.
     :param aggregator: function to aggregate scores of different folds (default: mean)
     :returns: a :class:`cross_validated_callable` with the proper configuration.
+
+    This resulting decorator must be used on a function with the following signature (+ potential other arguments):
+
+    :param x_train: training data
+    :type x_train: iterable
+    :param y_train: training labels (optional)
+    :type y_train: iterable
+    :param x_test: testing data
+    :type x_test: iterable
+    :param y_test: testing labels (optional)
+    :type y_test: iterable
+
+    `y_train` and `y_test` must be available of the `y` argument to this function is not None.
+
+    These 4 keyword arguments will be bound upon decoration.
+    Further arguments will remain free (e.g. hyperparameter names).
+
+    >>> data = list(range(5))
+    >>> @cross_validated(x=data, num_folds=5, folds=[[[i] for i in range(5)]], aggregator=identity)
+    ... def f(x_train, x_test, a):
+    ...     return x_test[0] + a
+    >>> f(a=1)
+    [1, 2, 3, 4, 5]
+    >>> f(1)
+    [1, 2, 3, 4, 5]
+    >>> f(a=2)
+    [2, 3, 4, 5, 6]
+
     """
+    args = dict(locals())
     def wrapper(f):
-        cv_callable = cross_validated_callable(f, x, num_folds, y, strata, folds,
-                                               num_iter, regenerate_folds, clusters,
-                                               aggregator)
-        return cv_callable
+        args['f'] = f
+        return cross_validated_callable(**args)
     return wrapper
 
 
 if __name__ == '__main__':
-    x = list(range(10))
-    clusters = [[1,2],[3,4,5]]
-
-    @cross_validated(x, num_folds=3, num_iter=2, clusters=clusters)
-    def f1(woops, x_train, x_test):
-        '''floopsie docstring'''
-        print(x_train)
-        return 0
-
-    print(f1.folds)
-    result = f1(woops='blah')
-    print(result)
-
-    def f(x, y, z):
-        return x + y + z
-
-    def f2(**kwargs):
-        kwargs['z'] = 2
-        return f(**kwargs)
-    print(str(f2(x=1, y=2)))
-
-    @cross_validated(list(range(20)), num_folds=10, num_iter=2, strata=[[1,2,3],[6,7,8,9]])
-    def f1(woops, x_train, x_test):
-        '''floopsie docstring'''
-        print(x_train)
-        return 0
+    pass
