@@ -1,7 +1,15 @@
-function [ stdin, stdout, stderr, p ] = popen( cmd, env )
+function [ in, out, p, socket ] = popen( cmd, env )
 %POPEN Spawns a subprocess p via Java API and enables bidirectional
-%communication with its stdin, stdout and stderr.
+%communication via sockets.
 
+%% create server socket
+serverSocket = java.net.ServerSocket(0);
+port = serverSocket.getLocalPort();
+
+% append port number to optunity's launch command
+cmd = [cmd, ' ', num2str(port)];
+
+%% launch Optunity Python back-end
 rt = java.lang.Runtime.getRuntime();
 if numel(env) > 0
     if iscell(env)
@@ -19,8 +27,9 @@ else
     p = rt.exec(java.lang.String(cmd));
 end
 
-stderr = java.io.BufferedReader(java.io.InputStreamReader(p.getErrorStream()));
-stdout = java.io.BufferedReader(java.io.InputStreamReader(p.getInputStream()));
-stdin = java.io.PrintWriter(p.getOutputStream());
+%% create communication socket and channels
+socket = serverSocket.accept();
+in = java.io.PrintWriter(socket.getOutputStream(), true);
+out = java.io.BufferedReader(java.io.InputStreamReader(socket.getInputStream()));
 
 end
\ No newline at end of file
